/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.ListCellRenderer;
import javax.swing.SwingUtilities;
import javax.swing.ToolTipManager;
import javax.swing.UIManager;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.plaf.ColorUIResource;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import com.eviware.soapui.model.DesktopPanel;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.PanelBuilder;
import com.eviware.soapui.model.project.Project;
import com.eviware.soapui.model.tree.SoapUITreeModel;
import com.eviware.soapui.model.tree.SoapUITreeNode;
import com.eviware.soapui.model.tree.SoapUITreeNodeRenderer;
import com.eviware.soapui.model.tree.nodes.ProjectTreeNode;
import com.eviware.soapui.model.workspace.Workspace;
import com.eviware.soapui.model.workspace.WorkspaceFactory;
import com.eviware.soapui.support.UISupport;
import com.jgoodies.looks.plastic.PlasticXPLookAndFeel;
import com.jgoodies.looks.plastic.theme.SkyBluerTahoma;

/**
 * Main SoapUI entry point.
 * 
 * @author Ole.Matzura
 * 
 * For version 0.6
 * @todo improve error assertion reporting +
 * @todo find/search/replace +
 * @todo undo +
 * @todo recreate request and keep values +
 * @todo dont create optional elements +
 * @todo option to create requests when importing +
 * @todo show soapaction +
 * @todo save response +
 * @todo color-coding +
 * @todo rapportera fel vid null-svar +
 * @todo fixa bugg vid update interface + 
 * @todo xmlbeans2 +
 * @todo httpclient rc3 +
 * @todo refactor request panels + 
 * @todo refactor project/interface/testsuite listeners +
 * @todo create properties table + 
 * 
 * For version 0.7
 * @todo workspace in home folder +
 * @todo allow renaming of operation names in tree +
 * @todo add functionality for copying values from test response to following test request +
 * @todo fix relative xsd imports when loading from file system +
 * @todo fix cancelling of request when closing request window -> introduce DesktopPanel interface +
 * @todo refactor panel creation / listener handling/release +
 * @todo fix update of testcase window when adding test steps +
 * @todo add delete-line action in xml-editor (Ctrl-D) +
 * @todo add submit action to xml request editor (Ctrl Enter) +
 * @todo add mousewheel support to xml editor +
 * @todo add mousewheel + C support to xml editor (move cursor) +
 * @todo fix revalidation problems +
 * @todo add validation to xmleditors +
 * @todo check localisation of schema errors + 
 * @todo add possibility to reload schema (neccessary?) +
 * @todo fix cloning of assertions when cloning test steps +
 * @todo ask for creation of optional elements when (re)creating new request +
 * @todo add action to add existing namespaces to xpath in xpathassertion +
 * 
 * For version 0.8
 * @todo fix default values when creating requests +
 * @todo bug-fixes
 * - creation of namespace declarations creates multiple with same name +
 * - check defintion update +
 * - deletion of endpoint always deletes first +
 * - deleted assertions result in error during save/exit? +
 * - disable submit when missing endpoint + 
 * - clone request null-pointer + 
 * - fix value transfer element <> attribute + 
 * - fix cancelling of test requests +
 * @todo create assertion events / listeners +
 * @todo ask to add testcase if missing +
 * @todo add alt-enter to response area +
 * @todo allow url config of SchemaCompliance assertion + 
 * @todo add accelerators for common actions + 
 * @todo fix positions of dialogs +
 * @todo add possibility to specify testsuite/testcase in maven-plugin +
 * @todo refactoring: replaced SoapUIAction with standard swing Action +
 * @todo refactoring: checked all JOptionPanes for correct title, etc. + 
 * @todo add filename-filters to filechoosers +
 * @todo add shortcut for cancel request to editor + 
 * 
 * For version 0.9
 * @todo initial class/code comments +
 * @todo bug-fixes
 * - fix cancelling of test runs +
 * - fix removal of request test-steps when associated operations / interfaces are removed + 
 * - update of request-step status in testcase panel + 
 * - focus on find-field in find/replace dialog +
 * - fix filters to not remove directories.. +
 * - change wsdl file filter to *.wsdl +
 * - disable declare in transfer panel when no transfer is selected +
 * - declare in transfer panel declares target from target response (should be request) +
 * - ask to cancel when closing running testcase +
 * - fix pretty-printing of large xml-documents +
 * - fix find and replace +
 * - fix node insert error on multiple clone teststep +
 * - fix log-messages from canceled requests +
 * - disable replace-related options in find/replace if editor is not editable +
 * @todo add possibility to move teststeps within testcase + 
 * @todo improve/disable disabling of request-editor +
 * @todo add addAssertion button to testrequest panel +
 * @todo add copy/pase actions to editor popup +
 * @todo model-items tests
 * @todo improve wsdl/xsd viewer +
 * @todo improve line-number reporting in validations +
 * @todo added shift-tab to move focus between request/response editors +
 * @todo refactorings
 * - SoapUIException +
 * - UIFacade for JOptionPane + (partially)
 * 
 * For version 1.0b
 * @todo initial user documentation / samples +
 * @todo bug-fixes +
 * @todo more tests / testing! +
 * @todo beta release +

 * For version 1.0
 * @todo official release +
 * 
 * 1.1 Improvements
 * @todo add possibility to transfer values from source request (instead of response)
 * @todo add possibility to move teststeps to another testcase
 * @todo workspace management
 * @todo add reload-project!?
 * @todo option to reload schemas on run testcase
 */

public class SoapUI
{
   // ------------------------------ FIELDS ------------------------------

   private static final String DEFAULT_WORKSPACE_FILE = "default-soapui-workspace.xml";

   private static SoapUI instance;

   private JFrame frame;

   private Map<ModelItem, JInternalFrame> modelItemToDesktopFrameMap = new HashMap<ModelItem, JInternalFrame>();
   private Map<ModelItem, DesktopPanel> modelItemToDesktopPanelMap = new HashMap<ModelItem, DesktopPanel>();

   private Map<JInternalFrame, ModelItem> contentPanelToModelItemMap = new HashMap<JInternalFrame, ModelItem>();

   private JTextArea logArea;
   
   private JDesktopPane desktopPane = new JDesktopPane();

   private JTree mainTree = new JTree();

   private SoapUITreeModel treeModel;

   private Workspace workspace;

   private static List<String> logCache = new ArrayList<String>();
   private static int openFrameCount = 0;
   private static final int xOffset = 30, yOffset = 30;

   private JMenu windowMenu;

   private JMenu fileMenu;

   private JMenuBar menuBar;
   
   private InternalDesktopFrameListener internalFrameListener = new InternalDesktopFrameListener();

   private JList desktopPanelsList;

   private DefaultListModel desktopPanels;

   private ModelItemPropertyChangeListener modelItemPropertyListener = new ModelItemPropertyChangeListener();

   private JSplitPane contentSplit;

	private JPanel overviewPanel;
	
	public final static String SOAPUI_VERSION = "1.0.1";

	private JTabbedPane detailTabs;

	private SoapUIDesktop desktop;
   

   // -------------------------- STATIC METHODS --------------------------

   static
   {
      try
      {
         PlasticXPLookAndFeel.setMyCurrentTheme(new SoapUITheme());
         PlasticXPLookAndFeel.setTabStyle("Metal");

         UIManager.setLookAndFeel(new PlasticXPLookAndFeel());
      }

      catch (Throwable e)
      {
         System.err.println( "Error initializing PlasticXPLookAndFeel; " + e.getMessage() );
      }
   }

   // --------------------------- CONSTRUCTORS ---------------------------

   public SoapUI()
   {
      ToolTipManager.sharedInstance().setInitialDelay( 200 );
      
      frame = new JFrame("SoapUI " + SOAPUI_VERSION);
      frame.addWindowListener(new WindowAdapter()
      {
         public void windowClosing(WindowEvent e)
         {
            if( JOptionPane.showConfirmDialog( frame, "Exit SoapUI?", "Question", JOptionPane.OK_CANCEL_OPTION )
                  != JOptionPane.OK_OPTION )
               return;
            
            workspace.onClose();
            frame.dispose();
         }

			public void windowClosed(WindowEvent e)
			{
				System.exit( 0 );
			}
         
      });

      UISupport.setMainFrame( frame );
      instance = this;
   }

   private void buildUI()
   {
      treeModel = new SoapUITreeModel(workspace);
      mainTree.setModel(treeModel);
      mainTree.setRootVisible(true);
      mainTree.setExpandsSelectedPaths(true);
      mainTree.setToggleClickCount( 0 );
      mainTree.addMouseListener(new TreeMouseListener());
      mainTree.addTreeSelectionListener(new InternalTreeSelectionListener());
      mainTree.setCellRenderer( new SoapUITreeNodeRenderer() );
      
      ToolTipManager.sharedInstance().registerComponent(mainTree);

      JSplitPane splitPane = UISupport.createHorizontalSplit( buildMainPanel(), buildContentPanel() ); 
      
      frame.setJMenuBar(buildMainMenu());
      frame.getContentPane().add(splitPane, BorderLayout.CENTER);
      frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
      frame.setSize(900, 700);

      splitPane.setDividerLocation(250);
   }

   public static SoapUI getInstance()
   {
      return instance;
   }

   private JMenuBar buildMainMenu()
   {
      menuBar = new JMenuBar();
      fileMenu = new JMenu("File");
      fileMenu.setMnemonic( KeyEvent.VK_F );
      
      Action[] actions = workspace.getActions();
      for (int i = 0; i < actions.length; i++)
      {
         fileMenu.add( actions[i] );
      }

      fileMenu.addSeparator();
      fileMenu.add( new SaveAction() );

      fileMenu.addSeparator();
      fileMenu.add( new ExitAction() );
       
      menuBar.add(fileMenu); 
      
      windowMenu = new JMenu( "Window");
      windowMenu.setMnemonic( KeyEvent.VK_W );
      windowMenu.add(new CloseCurrentAction());
      windowMenu.add(new CloseAllAction());
      windowMenu.add(new CloseOtherAction());
      windowMenu.addMenuListener( new MenuListener()
            {
               public void menuSelected(MenuEvent e)
               {
                  while( windowMenu.getMenuComponentCount() > 3 )
                     windowMenu.remove( 3 );
                  
                  if( contentPanelToModelItemMap.size() == 0 ) return;
                  windowMenu.addSeparator();
                  Iterator<JInternalFrame> i = contentPanelToModelItemMap.keySet().iterator();
                  while( i.hasNext() )
                  {
                     windowMenu.add( new PaneSelectionAction( i.next() ));
                  }
               }

               public void menuDeselected(MenuEvent e)
               {
               }

               public void menuCanceled(MenuEvent e)
               {
               }});

      menuBar.add(windowMenu); 
//      menuBar.add(new JMenu("About"));
       
      return menuBar;
   }

   public JFrame getFrame()
   {
      return frame;
   }

   private JComponent buildMainPanel()
   {
      JSplitPane splitPane = UISupport.createVerticalSplit();
      JPanel mainPanel = new JPanel(new BorderLayout());

      mainTree.setBorder(null);
      mainTree.getSelectionModel().setSelectionMode( TreeSelectionModel.SINGLE_TREE_SELECTION );
      splitPane.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));

      splitPane.setTopComponent(new JScrollPane(mainTree));
      splitPane.setBottomComponent(buildOverviewPanel());
      splitPane.setDividerLocation( 500 );
      splitPane.setResizeWeight( 0.6 );
      return splitPane;
   }

   private Component buildOverviewPanel()
   {
      detailTabs = new JTabbedPane( JTabbedPane.BOTTOM );
		desktopPanels = new DefaultListModel();
      desktopPanelsList = new JList( desktopPanels );
      desktopPanelsList.setCellRenderer( new DesktopItemsCellRenderer() );
      desktopPanelsList.setToolTipText( "Open windows" );
      desktopPanelsList.addMouseListener( new MouseAdapter() {

         public void mouseClicked(MouseEvent e)
         {
            if( e.getClickCount() < 2 ) return;
            ModelItem modelItem = (ModelItem) desktopPanelsList.getSelectedValue();
            if( modelItem == null ) return;
            
            showTab( modelItem );
         }} );
      
		JPanel panel = new JPanel( new BorderLayout() );
      panel.setBackground( Color.LIGHT_GRAY );
      panel.add( detailTabs, BorderLayout.CENTER );
      detailTabs.addTab( "Windows", new JScrollPane( desktopPanelsList ));

      overviewPanel = new JPanel( new BorderLayout() );
		detailTabs.addTab( "Details", overviewPanel );
      
      return panel;
   }

   private void setOverviewPanel(Component panel)
   {
   	if( overviewPanel.getComponentCount() == 0 && panel == null )
   		return;
   	
   	overviewPanel.removeAll();
   	if( panel != null )
   		overviewPanel.add( panel, BorderLayout.CENTER );
   	overviewPanel.revalidate();
   	overviewPanel.repaint();
   }

   private Component buildContentPanel()
   {
      contentSplit = UISupport.createVerticalSplit();
      JPanel contentPanel = new JPanel(new BorderLayout());

      contentSplit.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));

      contentSplit.setTopComponent(buildTabbedPanel());
      contentSplit.setBottomComponent(buildLogPanel());
      contentSplit.setDividerLocation(550);
      contentSplit.setResizeWeight( 0.9 );
      return contentSplit;
   }

   private class PaneSelectionAction extends AbstractAction
   {
      private final JInternalFrame frame2;

      public PaneSelectionAction(JInternalFrame frame)
      {
         super( frame.getTitle() );
         frame2 = frame;
      }

      public void actionPerformed(ActionEvent e)
      {
         showTab( contentPanelToModelItemMap.get( frame2 ));
      }}
   
   private Component buildTabbedPanel()
   {
      JScrollPane scrollPane = new JScrollPane( desktopPane );
      return scrollPane;
   }

   private Component buildLogPanel()
   {
      logArea = new JTextArea();
      logArea.setEditable(false);
      logArea.setToolTipText("SoapUI Log");
      
      for (String message : logCache)
      {
         logArea.setText( logArea.getText() + message + "\n" );
      }
      
      JPanel panel = new JPanel(new BorderLayout());
      panel.setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 3));
      panel.add(new JScrollPane(logArea), BorderLayout.CENTER);
      return panel;
   }

   // -------------------------- OTHER METHODS --------------------------

   public synchronized void log(final String msg)
   {
      if( instance == null || instance.logArea == null ) 
      {
         logCache.add( msg );
         return;
      }
      
      if (SwingUtilities.isEventDispatchThread())
      {
         instance.logArea.setText(instance.logArea.getText() + msg + "\r\n");
      }
      else
      {
         SwingUtilities.invokeLater(new Runnable()
         {
            public void run()
            {
               instance.logArea.setText(instance.logArea.getText() + msg + "\r\n");
            }
         });
      }
   }

   // -------------------------- INNER CLASSES --------------------------

   private static class SoapUITheme extends SkyBluerTahoma
   {
      public ColorUIResource getControl()
      {
         return new ColorUIResource(new Color(238, 237, 237));
      }
   }

   // --------------------------- main() method ---------------------------

   public static void main(String[] args) throws Exception
   {
      SoapUI soapUI = new SoapUI();
      Workspace workspace = null;

      if (args.length > 0)
         workspace = WorkspaceFactory.getInstance().openWorkspace(args);
      else
         workspace = WorkspaceFactory.getInstance().openWorkspace(new String[] { System.getProperty("user.home") + 
         		File.separatorChar + DEFAULT_WORKSPACE_FILE });

      soapUI.show(workspace);
   }

   private void show(Workspace workspace)
   {
      this.workspace = workspace;
      buildUI();

      desktop = new SoapUIDesktop( workspace );
      frame.setVisible(true);
   }

   public class TreeMouseListener extends MouseAdapter
   {
      public void mouseClicked(MouseEvent e)
      {
         if (e.isPopupTrigger())
            showPopup(e);
         else if (e.getClickCount() < 2)
            return;
         if (mainTree.getSelectionCount() != 1)
            return;

         TreePath path = mainTree.getSelectionPath();
         SoapUITreeNode node = (SoapUITreeNode) path.getLastPathComponent();
         
         showTab( node.getModelItem() );
      }

      public void mousePressed(MouseEvent e)
      {
         if (e.isPopupTrigger())
            showPopup(e);
      }

      public void mouseReleased(MouseEvent e)
      {
         if (e.isPopupTrigger())
            showPopup(e);
      }

      private void showPopup(MouseEvent e)
      {
         TreePath path = mainTree.getPathForLocation((int) e.getPoint().getX(), (int) e.getPoint().getY());
         if (path == null)
            return;
         SoapUITreeNode node = (SoapUITreeNode) path.getLastPathComponent();

         JPopupMenu popupMenu = node.getPopup();
         if (popupMenu == null)
            return;

         mainTree.setSelectionPath(path);

         popupMenu.show(mainTree, e.getX(), e.getY());
      }
   }

   public class CloseCurrentAction extends AbstractAction
   {
      public CloseCurrentAction()
      {
         super("Close Current");
         putValue(Action.SHORT_DESCRIPTION, "Closes the current window");
         putValue( Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke( KeyEvent.VK_R, KeyEvent.CTRL_MASK ));

      }

      public void actionPerformed(ActionEvent e)
      {
         JInternalFrame frame = desktopPane.getSelectedFrame();
         if( frame != null )
            closeTab( contentPanelToModelItemMap.get(frame));
      }
   }

   public class CloseOtherAction extends AbstractAction
   {
      public CloseOtherAction()
      {
         super("Close others");
         putValue(Action.SHORT_DESCRIPTION, "Closes all windows except the current one");
         putValue( Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke( KeyEvent.VK_O, KeyEvent.CTRL_MASK ));
         
      }

      public void actionPerformed(ActionEvent e)
      {
         JInternalFrame frame = desktopPane.getSelectedFrame();
         if( frame == null )
            return;

         Iterator<JInternalFrame> i = contentPanelToModelItemMap.keySet().iterator();
         while( i.hasNext() )
         {
            JInternalFrame f = i.next();
            if( f != frame ) 
            {
               closeTab( contentPanelToModelItemMap.get( f ) );
               i = contentPanelToModelItemMap.keySet().iterator();
            }
         }
      }
   }

   public class CloseAllAction extends AbstractAction
   {
      public CloseAllAction()
      {
         super("Close all");
         putValue( Action.SHORT_DESCRIPTION, "Closes all windows");
         putValue( Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke( KeyEvent.VK_L, KeyEvent.CTRL_MASK ));
      }

      public void actionPerformed(ActionEvent e)
      {
         while( contentPanelToModelItemMap.size() > 0 )
         {
            Iterator<JInternalFrame> i = contentPanelToModelItemMap.keySet().iterator();
            try
            {
               i.next().setClosed( true );
            }
            catch (PropertyVetoException e1)
            {
               e1.printStackTrace();
            }
         }
         
         contentPanelToModelItemMap.clear();
         modelItemToDesktopFrameMap.clear();
         modelItemToDesktopPanelMap.clear();
      }
   }

   public class InternalTreeSelectionListener implements TreeSelectionListener
   {
      public void valueChanged(TreeSelectionEvent e)
      {
         Object obj = e.getPath().getLastPathComponent();
         if (obj instanceof SoapUITreeNode)
         {
            SoapUITreeNode treeNode = (SoapUITreeNode) obj;
            setOverviewPanel(treeNode.getOverviewPanel());
         }
      }
   }

   public Workspace getWorkspace()
   {
      return workspace;
   }
   
   public boolean hasTab( ModelItem modelItem )
   {
      return modelItemToDesktopFrameMap.containsKey( modelItem );
   }
   
   public JInternalFrame showTab( ModelItem modelItem )
   {
      PanelBuilder panelBuilder = modelItem.getPanelBuilder();
      if (modelItemToDesktopFrameMap.containsKey(modelItem))
      {
         JInternalFrame frame = modelItemToDesktopFrameMap.get(modelItem);
         try
         {
            desktopPane.getDesktopManager().deiconifyFrame( frame );
            frame.setSelected( true );
            frame.moveToFront();
            return frame;
         }
         catch (PropertyVetoException e)
         {
            e.printStackTrace();
         }
      }
      else if (panelBuilder != null && panelBuilder.canBuildDesktopPanel())
      {
         String title = panelBuilder.getDesktopPanelTitle();
         DesktopPanel desktopPanel = panelBuilder.buildDesktopPanel();
			JComponent panel = desktopPanel.getComponent();
         panel.setOpaque( true );
         
         JInternalFrame frame = createContentFrame(title, panel);

         desktopPane.add( frame );
         try
         {
            frame.setSelected( true );
         }
         catch (PropertyVetoException e)
         {
            e.printStackTrace();
         }
         
         modelItemToDesktopFrameMap.put(modelItem, frame);
         modelItemToDesktopPanelMap.put( modelItem, desktopPanel );
         contentPanelToModelItemMap.put(frame, modelItem);
         
         desktopPanels.addElement( modelItem );
         desktopPanelsList.setSelectedValue( modelItem, false );
         modelItem.addPropertyChangeListener( modelItemPropertyListener );
         
         return frame;
      }
      else Toolkit.getDefaultToolkit().beep();
      
      return null;
   }

   private JInternalFrame createContentFrame(String title, JComponent panel)
   {
      JInternalFrame frame = new JInternalFrame(title, true, true, true, true );
      frame.addInternalFrameListener( internalFrameListener );
      frame.setContentPane( panel );
      frame.setSize( panel.getPreferredSize() );
      frame.setVisible( true );
      frame.setDefaultCloseOperation( JInternalFrame.DO_NOTHING_ON_CLOSE );
      frame.setLocation(xOffset*(openFrameCount%10), yOffset*(openFrameCount%10));
      frame.setBorder( BorderFactory.createCompoundBorder(
            BorderFactory.createRaisedBevelBorder(), BorderFactory.createEmptyBorder( 2, 2, 2, 2 )));
      openFrameCount++;
      return frame;
   }

   public void closeTab(ModelItem modelItem)
   {
      if (modelItemToDesktopFrameMap.containsKey(modelItem))
      {
      	JInternalFrame frame = modelItemToDesktopFrameMap.get(modelItem);
      	frame.doDefaultCloseAction();
      }
   }
   
   public Project getCurrentProject()
   {
      TreePath path = mainTree.getSelectionPath();
      Object node = (Object) path.getLastPathComponent();
      while (node != null && !(node instanceof ProjectTreeNode))
      {
         path = path.getParentPath();
         node = (path == null ? null : path.getLastPathComponent());
      }

      if (node == null)
         return null;

      return ((ProjectTreeNode) node).getProject();
   }
   
   /** Returns an ImageIcon, or null if the path was invalid. */
   public static ImageIcon createImageIcon(String path) {
       java.net.URL imgURL = SoapUI.class.getResource(path);
       if (imgURL != null) {
           return new ImageIcon(imgURL);
       } else {
           System.err.println("Couldn't find icon file: " + path);
           return null;
       }
   }
   
   private class ExitAction extends AbstractAction
   {
      public ExitAction()
      {
         super( "Exit" );
         putValue( Action.SHORT_DESCRIPTION, "Saves all projects and exits SoapUI" );
         putValue( Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke( KeyEvent.VK_Q, KeyEvent.CTRL_MASK ));
      }

      public void actionPerformed(ActionEvent e)
      {
         frame.dispatchEvent( new WindowEvent( frame, WindowEvent.WINDOW_CLOSING ));
      }
   }

   private class SaveAction extends AbstractAction
   {
      public SaveAction()
      {
         super( "Save all" );
         putValue( Action.SHORT_DESCRIPTION, "Saves all projects" );
         putValue( Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke( KeyEvent.VK_S, KeyEvent.CTRL_MASK ));
      }

      public void actionPerformed(ActionEvent e)
      {
         workspace.save();
      }
   }
   
   
   private class InternalDesktopFrameListener extends InternalFrameAdapter
   {
      public void internalFrameClosing(InternalFrameEvent e)
      {
      	ModelItem modelItem = contentPanelToModelItemMap.get( e.getInternalFrame() );
      	DesktopPanel desktopPanel = modelItemToDesktopPanelMap.get( modelItem );
      	if( !desktopPanel.onClose() )
      	{
      		return;
      	}
      	
         modelItemToDesktopFrameMap.remove( modelItem );
         modelItemToDesktopPanelMap.remove( modelItem );
         contentPanelToModelItemMap.remove( e.getInternalFrame() );
         desktopPanels.removeElement( modelItem );
         
         e.getInternalFrame().dispose();
      }

      public void internalFrameActivated(InternalFrameEvent e)
      {
         ModelItem modelItem = contentPanelToModelItemMap.get( e.getInternalFrame() );
         if( modelItem != null )
         {
            desktopPanelsList.setSelectedValue( modelItem, false );
            selectModelItem( modelItem );
         }
      }
   }

   private class DesktopItemsCellRenderer extends JLabel implements ListCellRenderer 
   {
      public Component getListCellRendererComponent(
        JList list,
        Object value,           
        int index,               
        boolean isSelected,      
        boolean cellHasFocus)    
      {
          ModelItem modelItem = (ModelItem) value;
          
          String title = modelItemToDesktopFrameMap.get( modelItem ).getTitle();
          setText( title );
          
          setToolTipText( title );
          setIcon( modelItem.getIcon() );
          
          if (isSelected) 
          {
             setBackground(list.getSelectionBackground());
             setForeground(list.getSelectionForeground());
          }
          else 
          {
             setBackground(list.getBackground());
             setForeground(list.getForeground());
          }
          
          setEnabled(list.isEnabled());
          setFont(list.getFont());
          setOpaque(true);
          
          return this;
      }
   }
   
   private class ModelItemPropertyChangeListener implements PropertyChangeListener
   {
      public void propertyChange(PropertyChangeEvent evt)
      {
         ModelItem modelItem = (ModelItem) evt.getSource();
         int ix = desktopPanels.indexOf( modelItem );
         if( ix >= 0 )
            desktopPanels.set( ix, modelItem );
      }}

   public void selectModelItem(ModelItem modelItem)
   {
      TreePath path = treeModel.getPath( modelItem );
      mainTree.setSelectionPath( path );
      mainTree.scrollPathToVisible( path );
   }

   public SoapUITreeModel getTreeModel()
   {
      return treeModel;
   }

   public DesktopPanel[] getDesktopPanels()
	{
		return modelItemToDesktopPanelMap.values().toArray( new DesktopPanel[modelItemToDesktopPanelMap.size()] );
	}
}
