/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.request;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.AbstractListModel;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.ListCellRenderer;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.actions.request.CreateEmptyRequestAction;
import com.eviware.soapui.impl.wsdl.actions.request.RecreateRequestAction;
import com.eviware.soapui.impl.wsdl.panels.request.actions.ValidateRequestXmlAction;
import com.eviware.soapui.impl.wsdl.panels.request.actions.ValidateResponseXmlAction;
import com.eviware.soapui.impl.wsdl.panels.request.support.EndpointsComboBoxModel;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlAssertion;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequest;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStep;
import com.eviware.soapui.impl.wsdl.teststeps.actions.AddAssertionAction;
import com.eviware.soapui.impl.wsdl.teststeps.actions.CloneTestRequestStepAction;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError;
import com.eviware.soapui.model.DesktopPanel;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.iface.Submit;
import com.eviware.soapui.model.iface.SubmitListener;
import com.eviware.soapui.model.iface.Submit.Status;
import com.eviware.soapui.support.JXmlTextArea;
import com.eviware.soapui.support.SoapUISplitPaneUI;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.actions.ChangeSplitPaneOrientationAction;
import com.eviware.soapui.support.actions.FormatXmlAction;
import com.eviware.soapui.support.actions.SaveXmlTextAreaAction;
import com.jgoodies.forms.builder.ButtonBarBuilder;

/**
 * DesktopPanel for WsdlTestRequest. Essentially a copy of WsdlRequestDesktopPanel with assertions.
 * 
 * @author Ole.Matzura
 */

public class WsdlTestRequestDesktopPanel extends JPanel implements DesktopPanel
{
   private JXmlTextArea resultArea;
   private JXmlTextArea inputArea;
   private JComboBox endpointCombo;
   private JButton submitButton;
   private JButton recreateButton;
   private JButton cancelButton;
   private JComboBox encodingCombo;
   private WsdlTestRequest request;
   private EndpointsComboBoxModel endpointsModel;
   
   private final static Log log = LogFactory.getLog( WsdlTestRequestDesktopPanel.class );
   private JTextArea logArea;
   private JButton createEmptyButton;
   private JList assertionList;
   private JLabel statusLabel;
   private JPopupMenu assertionListPopup;
   private AssertionListModel assertionListModel;
   private InternalPropertyChangeListener internalPropertyChangeListener;
   private JButton cloneButton;
   private JButton splitButton;
   private WsdlTestRequestStep requestStep;
	public Submit submit;
	private InternalSubmitListener internalSubmitListener;
	private JSplitPane requestSplitPane;
	private MoveFocusAction moveFocusAction;
	private JButton addAssertionButton;

   public WsdlTestRequestDesktopPanel( WsdlTestRequestStep requestStep )
   {
      super( new BorderLayout() );
      this.requestStep = requestStep;
      this.request = requestStep.getTestRequest();
      this.endpointsModel = new EndpointsComboBoxModel( request );

      internalSubmitListener = new InternalSubmitListener();
		request.addSubmitListener( internalSubmitListener );
      
      internalPropertyChangeListener = new InternalPropertyChangeListener();
      request.addPropertyChangeListener( internalPropertyChangeListener );
      request.getOperation().getInterface().addPropertyChangeListener( internalPropertyChangeListener ); 
      
      add( buildUI(), BorderLayout.CENTER );
      add( buildSearchBar(), BorderLayout.NORTH );
      
      statusLabel = new JLabel();
      add( statusLabel, BorderLayout.PAGE_END );
      
      setPreferredSize( new Dimension( 600, 500 ));
   }

   private Component buildLogPanel()
   {
      JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.BOTTOM );
      
      logArea = new JTextArea();
      logArea.setEditable(false);
      logArea.setToolTipText("Request Log");
      
      JPanel panel = new JPanel(new BorderLayout());
      panel.add(new JScrollPane(logArea), BorderLayout.CENTER);
      
      tabbedPane.addTab( "Assertions", buildAssertionsPanel() );
      tabbedPane.addTab( "Request Log", panel );
      return tabbedPane;
   }
   
   private Component buildAssertionsPanel()
   {
      assertionListModel = new AssertionListModel();
      assertionList = new JList( assertionListModel );
      assertionList.setToolTipText( "Assertions for this request" );
      assertionList.setCellRenderer( new AssertionCellRenderer() );
      
      assertionListPopup = new JPopupMenu();
      assertionListPopup.add( new AddAssertionAction( requestStep.getTestRequest() ));
      
      assertionListPopup.addPopupMenuListener( new PopupMenuListener(){

         public void popupMenuWillBecomeVisible(PopupMenuEvent e)
         {
            while( assertionListPopup.getComponentCount() > 1 ) 
               assertionListPopup.remove( 1 );
            
            if( assertionListModel.getSize() == 0 ) return;
            int ix = assertionList.getSelectedIndex();
            if( ix == -1 ) return;
            
            WsdlAssertion assertion = assertionListModel.getAssertionAt( ix );
            Action[] actions = assertion.getActions();
            
            for (int i = 0; i < actions.length; i++)
            {
               assertionListPopup.add( actions[i] );
            }
         }

         public void popupMenuWillBecomeInvisible(PopupMenuEvent e)
         {
         }

         public void popupMenuCanceled(PopupMenuEvent e)
         {
         }});
      
      assertionList.setComponentPopupMenu( assertionListPopup );
      
      assertionList.addMouseListener( new MouseAdapter() {

			public void mouseClicked(MouseEvent e)
         {
         	
            if( e.getClickCount() < 2 ) return;

            int ix = assertionList.getSelectedIndex();
            if( ix == -1 ) return;
            
            Object obj = assertionList.getModel().getElementAt( ix );
            if( obj instanceof WsdlAssertion )
            {
	            WsdlAssertion assertion = (WsdlAssertion) obj;
	            if( assertion.isConfigurable() )
	               assertion.configure();
            }
   		   if( obj instanceof AssertionError )
   		   {
   		   	AssertionError error = (AssertionError) obj;
   		   	if( error.getLineNumber() >= 0 )
   		   	{
   		   		resultArea.setCaretPosition( resultArea.getLineStartOffset( error.getLineNumber()-1 ) );
   		   		resultArea.requestFocus();
   		   	}
   		   	else Toolkit.getDefaultToolkit().beep();
   		   }
   		   else Toolkit.getDefaultToolkit().beep();
         }});
      
      return new JScrollPane( assertionList );
   }

   class AssertionCellRenderer extends JLabel implements ListCellRenderer 
   {
      public Component getListCellRendererComponent(
        JList list,
        Object value,           
        int index,               
        boolean isSelected,      
        boolean cellHasFocus)    
      {
      	if( value instanceof WsdlAssertion )
      	{
      		WsdlAssertion assertion = (WsdlAssertion) value;
      		setText( assertion.getName() + " - " + assertion.getStatus().toString() );
            setIcon( assertion.getIcon() );
      	}
      	else if( value instanceof AssertionError )
      	{
            AssertionError assertion = (AssertionError) value;
            setText( " -> " + assertion.toString() );
            setIcon( null );
      	}
      	
          if (isSelected) 
          {
             setBackground(list.getSelectionBackground());
             setForeground(list.getSelectionForeground());
          }
          else 
          {
             setBackground(list.getBackground());
             setForeground(list.getForeground());
          }
          
          setEnabled(list.isEnabled());
          setFont(list.getFont());
          setOpaque(true);
          
          return this;
      }
  }
   
   private class AssertionListModel extends AbstractListModel implements PropertyChangeListener, 
       WsdlTestRequest.WsdlTestRequestListener
   {
   	private List<Object> items = new ArrayList<Object>();
   	
      public AssertionListModel()
      {
      	init();
      }
      
      public int getSize()
      {
      	return items.size();
      }

      public Object getElementAt(int index)
      {
      	return items.get( index );
      }

      public WsdlAssertion getAssertionAt( int index )
      {
      	Object object = items.get( index );
      	while( object instanceof AssertionError && index > 0 )
      	{
      		object = items.get( --index );
      	}
      	
      	return (WsdlAssertion) object;
      }
      
      public void refresh()
      {
			release();
      	init();
         fireContentsChanged( this, 0, getSize() );
      }

		private void init()
		{
      	request.addWsdlTestRequestListener( this );
			
         for( int c = 0; c < request.getAssertionCount(); c++ )
         {
         	WsdlAssertion assertion = request.getAssertionAt( c );
         	addAssertion(assertion);
         }
		}
		
		public void release()
		{
         items.clear();

         for( int c = 0; c < request.getAssertionCount(); c++ )
         {
         	WsdlAssertion assertion = request.getAssertionAt( c );
         	assertion.removePropertyChangeListener( this );
         }
         
         request.removeWsdlTestRequestListener( this );
		}

		public void propertyChange(PropertyChangeEvent evt)
		{
			refresh();
		}

		public void assertionAdded(WsdlAssertion assertion)
		{
			int sz = getSize();
      	addAssertion(assertion);
      	
      	fireIntervalAdded( this, sz, items.size()-1 );
		}

		private void addAssertion(WsdlAssertion assertion)
		{
			assertion.addPropertyChangeListener( this );
      	items.add( assertion );
      	
      	AssertionError[] errors = assertion.getErrors();
      	if( errors != null)
      	{
      		for( int i = 0; i < errors.length; i++ )
      		items.add( errors[i] );
      	}
		}

		public void assertionRemoved(WsdlAssertion assertion)
		{
			int ix = items.indexOf( assertion );
			if( ix == -1 ) return;

			assertion.removePropertyChangeListener( this );
			items.remove( ix );
			fireIntervalRemoved( this, ix, ix );
			
			// remove associated errors
			while( ix < items.size() && items.get( ix ) instanceof AssertionError )
			{
				items.remove( ix );
				fireIntervalRemoved( this, ix, ix );
			}
		}
   }
   
   private JComponent buildUI()
   {
   	requestSplitPane = UISupport.createHorizontalSplit();
		submitButton = createActionButton( new SubmitAction(), true );
      cancelButton = createActionButton( new CancelAction(), false );
      addAssertionButton = createActionButton( new AddAssertionAction( request ), true );
      splitButton = createActionButton( new ChangeSplitPaneOrientationAction( requestSplitPane ), true );
      recreateButton = createActionButton( new RecreateRequestAction(request), true );
      cloneButton = createActionButton( new CloneTestRequestStepAction( requestStep ), true );
      createEmptyButton = createActionButton( new CreateEmptyRequestAction(request), true );
      
      submitButton.setEnabled( request.getEndpoint() != null && request.getEndpoint().trim().length() > 0 );
      moveFocusAction = new MoveFocusAction();
      
      requestSplitPane.setTopComponent( buildInputUI() );
      requestSplitPane.setBottomComponent( buildOutputUI() );
      requestSplitPane.setDividerLocation( 0.5 );
      requestSplitPane.setResizeWeight( 0.5 );
      requestSplitPane.setBorder( null );
      
      JSplitPane outerSplit = UISupport.createVerticalSplit();
      outerSplit.setTopComponent( requestSplitPane );
      outerSplit.setBottomComponent( buildLogPanel() );
      outerSplit.setDividerLocation( 350 );
      outerSplit.setResizeWeight( 0.9 );
      outerSplit.setBorder( null );

      return outerSplit;
   }
   
   private JButton createActionButton( Action action, boolean enabled )
   {
   	JButton button = new JButton( action );
   	action.putValue( Action.NAME, null );
   	button.setPreferredSize( new Dimension(24, 24 ));
   	button.setEnabled( enabled );
   	return button;
   }

   private Component buildOutputUI()
   {
   	resultArea = new JXmlTextArea();
      resultArea.setMinimumSize( new Dimension( 50, 50 ));
      resultArea.setToolTipText( "Response Area" );
      if( request.getResponseContent() != null )
      {
   		resultArea.setText( request.getResponseContent() );
   		resultArea.setCaretPosition( 0 );
      }
      resultArea.addFocusListener( new ResultAreaFocusListener() );
      
      DefaultListModel listModel = new DefaultListModel();
      JList list = new JList( listModel );
      list.setToolTipText( "Response Validation Errors" );
      list.addMouseListener( new ValidationListMouseAdapter( list, resultArea ));
		JScrollPane scrollPane = new JScrollPane( list );
		scrollPane.setVisible( false );
		
		JSplitPane resultAreaSplit = UISupport.createVerticalSplit();

		JPopupMenu popup = new JPopupMenu();
      ValidateResponseXmlAction validateResponseXmlAction = new ValidateResponseXmlAction( request, scrollPane, listModel, resultAreaSplit, resultArea );
      popup.add( validateResponseXmlAction );
      FormatXmlAction formatXmlAction = new FormatXmlAction(resultArea);
		popup.add( formatXmlAction);
      popup.addSeparator();
      popup.add( resultArea.getUndoAction() );
      popup.add( resultArea.getRedoAction() );
      popup.add( resultArea.getCopyAction() );
      popup.add( resultArea.getCutAction() );
      popup.add( resultArea.getPasteAction() );
      popup.addSeparator();
      popup.add( resultArea.getFindAndReplaceAction() );
      popup.addSeparator();
      SaveXmlTextAreaAction saveXmlTextAreaAction = new SaveXmlTextAreaAction(resultArea, "Save Response");
		popup.add( saveXmlTextAreaAction);
      
      resultArea.setRightClickPopup( popup );
      resultArea.getInputHandler().addKeyBinding( "A+ENTER", submitButton.getAction() );
      resultArea.getInputHandler().addKeyBinding( "A+X", cancelButton.getAction() );
      resultArea.getInputHandler().addKeyBinding( "A+V", validateResponseXmlAction );
      resultArea.getInputHandler().addKeyBinding( "A+F", formatXmlAction );
      resultArea.getInputHandler().addKeyBinding( "A+S", saveXmlTextAreaAction );
      resultArea.getInputHandler().addKeyBinding( "S+TAB", moveFocusAction );
      
      resultAreaSplit.setTopComponent( resultArea );
		resultAreaSplit.setBottomComponent( scrollPane);
      resultAreaSplit.setDividerLocation( 1.0 );
      
      return resultAreaSplit;
   }

   private Component buildInputUI()
   {
      inputArea = new JXmlTextArea();
      inputArea.setMinimumSize( new Dimension( 50, 50 ));
      inputArea.setToolTipText( "Request Area" );
      inputArea.setText( request.getRequestContent() );
      inputArea.setCaretPosition( 0 );
      inputArea.setDiscardEditsOnSet( false );
      inputArea.addFocusListener( new InputAreaFocusListener());
      
      DefaultListModel listModel = new DefaultListModel();
      JList list = new JList( listModel );
      list.setToolTipText( "Request Validation Errors" );
      list.addMouseListener( new ValidationListMouseAdapter( list, inputArea ));
		JScrollPane scrollPane = new JScrollPane( list );
		scrollPane.setVisible( false );
		
		JSplitPane inputAreaSplit = UISupport.createVerticalSplit();
      
      JPopupMenu inputPopup = new JPopupMenu();
      ValidateRequestXmlAction validateRequestXmlAction = new ValidateRequestXmlAction( request, scrollPane, listModel, inputAreaSplit, inputArea );
		inputPopup.add( validateRequestXmlAction);
      FormatXmlAction formatXmlAction = new FormatXmlAction(inputArea);
		inputPopup.add( formatXmlAction);
      inputPopup.addSeparator();
      inputPopup.add( inputArea.getUndoAction() );
      inputPopup.add( inputArea.getRedoAction() );
      inputPopup.add( inputArea.getCopyAction() );
      inputPopup.add( inputArea.getCutAction() );
      inputPopup.add( inputArea.getPasteAction() );
      inputPopup.addSeparator();
      inputPopup.add( inputArea.getFindAndReplaceAction() );
      inputPopup.addSeparator();
      SaveXmlTextAreaAction saveXmlTextAreaAction = new SaveXmlTextAreaAction(inputArea, "Save Request");
		inputPopup.add( saveXmlTextAreaAction);
		
      inputArea.setRightClickPopup( inputPopup );
      
      inputArea.getDocument().addDocumentListener( new DocumentListener() {

         public void insertUpdate(DocumentEvent e)
         {
            request.setRequest( inputArea.getText() );
         }

         public void removeUpdate(DocumentEvent e)
         {
            request.setRequest( inputArea.getText() );
         }

         public void changedUpdate(DocumentEvent e)
         {
            
         }} );
      
      inputArea.getInputHandler().addKeyBinding( "A+ENTER", submitButton.getAction() );
      inputArea.getInputHandler().addKeyBinding( "A+X", cancelButton.getAction() );
      inputArea.getInputHandler().addKeyBinding( "A+V", validateRequestXmlAction );
      inputArea.getInputHandler().addKeyBinding( "A+F", formatXmlAction );
      inputArea.getInputHandler().addKeyBinding( "A+S", saveXmlTextAreaAction );
      inputArea.getInputHandler().addKeyBinding( "S+TAB", moveFocusAction );
     
      inputAreaSplit.setTopComponent( inputArea );
		inputAreaSplit.setBottomComponent( scrollPane);
      inputAreaSplit.setDividerLocation( 1.0 );
      
      return inputAreaSplit;
   }

   private JComponent buildSearchBar()
   {
      endpointCombo = new JComboBox( endpointsModel);
      endpointCombo.setToolTipText( endpointsModel.getSelectedItem().toString() );
      endpointCombo.setPreferredSize( new Dimension( 300, 20 ));
      
      endpointsModel.setSelectedItem( request.getEndpoint());
      
      endpointCombo.addItemListener( new ItemListener() {

         public void itemStateChanged(ItemEvent e)
         {
         	Object item = endpointCombo.getSelectedItem();
            if( item == null )
            {
            	endpointCombo.setToolTipText( "- no endpoint set for request -" );
            }
            else
            {
					String selectedItem = item.toString();
	            endpointCombo.setToolTipText( selectedItem );
            }
         
            submitButton.setEnabled( submit == null && request.getEndpoint() != null && request.getEndpoint().trim().length() > 0 );
         }});
      
      encodingCombo = new JComboBox();
      encodingCombo.setEditable( true );
      encodingCombo.addItem( "UTF-8" );
      encodingCombo.addItem( "iso-8859-1" );
      encodingCombo.setSelectedItem( request.getEncoding() );
      encodingCombo.setToolTipText( "Sets the encoding to use for the request" );
      
      encodingCombo.addItemListener( new ItemListener() {

         public void itemStateChanged(ItemEvent e)
         {
            request.setEncoding( encodingCombo.getSelectedItem().toString() );
         }});
      
      ButtonBarBuilder builder = new ButtonBarBuilder();
      builder.addFixed( submitButton );
      builder.addRelatedGap();
      builder.addFixed( addAssertionButton );
      builder.addRelatedGap();
      builder.addFixed( recreateButton );
      builder.addRelatedGap();
      builder.addFixed( createEmptyButton );
      builder.addRelatedGap();
      builder.addFixed( cloneButton );
      builder.addRelatedGap();
      builder.addFixed( cancelButton );
      builder.addUnrelatedGap();
      builder.addFixed( endpointCombo );
      builder.addRelatedGap();
      builder.addFixed( encodingCombo );
      builder.addRelatedGap();
      builder.addFixed( splitButton );


      builder.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
      return builder.getPanel();
   }

   public void setEnabled( boolean enabled )
   {
      endpointCombo.setEnabled( enabled );
      inputArea.setEditable( enabled );
      resultArea.setEditable( enabled );
      addAssertionButton.setEnabled( enabled );
      submitButton.setEnabled( enabled && request.getEndpoint() != null && request.getEndpoint().trim().length() > 0);
      recreateButton.setEnabled( enabled );
      createEmptyButton.setEnabled( enabled );
      cloneButton.setEnabled( enabled );
      encodingCombo.setEnabled( enabled );
   }
   
   public class ValidateAction extends AbstractAction
   {
      public ValidateAction()
      {
         putValue( Action.SMALL_ICON, SoapUI.createImageIcon( "/validate_request.gif"));
         putValue( Action.SHORT_DESCRIPTION, "Validates the current response with the current assertions" );
      }

      public void actionPerformed( ActionEvent e )
      {
         request.setResponse( resultArea.getText() );
      }
   }
   
   public class SubmitAction extends AbstractAction
   {
      public SubmitAction()
      {
         putValue( Action.SMALL_ICON, SoapUI.createImageIcon( "/submit_request.gif"));
         putValue( Action.SHORT_DESCRIPTION, "Submit request to specified endpoint URL" );
         putValue( Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke( KeyEvent.VK_ENTER, KeyEvent.ALT_MASK ));
      }

      public void actionPerformed( ActionEvent e )
      {
      	if( submit != null && submit.getStatus() == Submit.Status.RUNNING )
      	{
      		if( UISupport.confirm( "Cancel current request?", "Submit Request" ))
      		{
      			submit.cancel();
      		}
      		else return;
      	}
      	
      	submit = request.submit();
      }
   }

   private class CancelAction extends AbstractAction
   {
      public CancelAction()
      {
         super();
         putValue( Action.SMALL_ICON, SoapUI.createImageIcon( "/cancel_request.gif"));
         putValue( Action.SHORT_DESCRIPTION, "Aborts ongoing request" );
         putValue( Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke( KeyEvent.VK_X, KeyEvent.ALT_MASK ));
      }

      public void actionPerformed( ActionEvent e )
      {
      	if( submit == null ) return;
      	
         cancelButton.setEnabled( false );
         WsdlTestRequestDesktopPanel.this.setEnabled( true );
         submit.cancel();
      }
   }

   private class InternalSubmitListener implements SubmitListener
   {
      private long startTime;
      private SimpleDateFormat sdf;

      private InternalSubmitListener()
      {
         sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
      }

      public boolean beforeSubmit(Submit submit)
      {
      	if( submit.getRequest() != request ) return true;
      	
         setEnabled( false );
         cancelButton.setEnabled( WsdlTestRequestDesktopPanel.this.submit != null );
         startTime = System.currentTimeMillis();
         return true;
      }

      public void afterSubmit(Submit submit)
      {
      	if( submit.getRequest() != request ) return;
      	Status status = submit.getStatus();
      	
         if( status != Status.CANCELED )
         {
            cancelButton.setEnabled( false );
            setEnabled( true );
            
            request.setResponse( submit.getResponse( false ).getContentAsString() );
         }
         
         String message = null;
         String infoMessage = null;
         String requestName = request.getOperation().getInterface().getName() + "." + 
            request.getOperation().getName() + ":" + request.getName();

         if( status == Status.CANCELED ) 
         {
            message = "CANCELED";
            infoMessage = "[" + requestName + "] - CANCELED";
         }
         else
         {
            if( status == Status.ERROR )
            {
               message = "Error getting response; " + submit.getError().getMessage();
               infoMessage = "Error getting response for [" + requestName + "]; " + submit.getError().getMessage();
            }
            else
            {
               message = "request time: " + submit.getTimeTaken() + "ms (" + 
                  submit.getResponse( false ).getContentLength() + " bytes)";   
               infoMessage = "Got response for [" + requestName + "] in " + submit.getTimeTaken() + "ms (" + 
                  submit.getResponse( false ).getContentLength() + " bytes)";   
               
               resultArea.requestFocus();
            }
         }
         
         log.info( infoMessage );
         
         String logText = logArea.getText() + sdf.format( new Date( startTime )) + " - " + message + "\r\n";
         logArea.setText( logText );
         logArea.setCaretPosition( logText.length() );
         statusLabel.setText( message  );
         
         WsdlTestRequestDesktopPanel.this.submit = null;
      }
   }
   
   private class InternalPropertyChangeListener implements PropertyChangeListener
   {
      public void propertyChange(PropertyChangeEvent evt)
      {
         String propertyName = evt.getPropertyName();
         if( propertyName.equals( WsdlRequest.RESPONSE_PROPERTY ))
         {
            resultArea.setText( request.getResponseContent() );
            resultArea.setCaretPosition( 0 );
         }
         else if( propertyName.equals( WsdlRequest.ENCODING_PROPERTY ))
         {
            encodingCombo.setSelectedItem( request.getEncoding() );
         }
         else if( propertyName.equals( WsdlTestRequest.STATUS_PROPERTY ))
         {
            assertionListModel.refresh(); 
         }
         else if( propertyName.equals( WsdlRequest.REQUEST_PROPERTY ))
         {
         	String requestContent = request.getRequestContent();
            if( !requestContent.equals( inputArea.getText()))
            	inputArea.setText( requestContent );
         }
      }
   }
   
   private class MoveFocusAction extends AbstractAction
   {
		public void actionPerformed(ActionEvent e)
		{
			if( inputArea.hasFocus() )
				resultArea.requestFocus();
			else 
			   inputArea.requestFocus();
		}
   }
   
   private final class InputAreaFocusListener implements FocusListener
	{
		public void focusGained(FocusEvent e)
		{
			// dont resize if split has been dragged 
			if( ((SoapUISplitPaneUI)requestSplitPane.getUI()).hasBeenDragged() ) return;
			
			int pos = requestSplitPane.getDividerLocation();
			if( pos >= 600 ) return;
			if( requestSplitPane.getMaximumDividerLocation() > 700 )
				requestSplitPane.setDividerLocation( 600 );
			else
			   requestSplitPane.setDividerLocation( 0.8 );
		}

		public void focusLost(FocusEvent e)
		{
		}
	}
   
   private final class ResultAreaFocusListener implements FocusListener
	{
		public void focusGained(FocusEvent e)
		{
			if( ((SoapUISplitPaneUI)requestSplitPane.getUI()).hasBeenDragged() || resultArea.getText().trim().length() == 0 ) 
				return;
			
			int pos = requestSplitPane.getDividerLocation();
			int maximumDividerLocation = requestSplitPane.getMaximumDividerLocation();
			if( pos + 600 < maximumDividerLocation ) return;
			
			if( maximumDividerLocation > 700 )
				requestSplitPane.setDividerLocation( maximumDividerLocation-600 );
			else
				requestSplitPane.setDividerLocation( 0.2 );
		}

		public void focusLost(FocusEvent e)
		{
		}
	}

   public ModelItem getModelItem()
	{
		return requestStep;
	}

	public boolean onClose()
	{
		if( submit != null && submit.getStatus() == Submit.Status.RUNNING )
		{
			int retVal = JOptionPane.showConfirmDialog( SoapUI.getInstance().getFrame(), "Cancel request before closing?",
					 "Closing window", JOptionPane.YES_NO_CANCEL_OPTION);
			if( retVal == JOptionPane.CANCEL_OPTION ) return false;
			
			if( retVal == JOptionPane.YES_OPTION && submit.getStatus() == Submit.Status.RUNNING )
				submit.cancel();
		}
		
      request.removeSubmitListener( internalSubmitListener );
      request.removePropertyChangeListener( internalPropertyChangeListener );
      request.getOperation().getInterface().removePropertyChangeListener( internalPropertyChangeListener );
      assertionListModel.release();
      endpointsModel.release();
      
      return true;
	}

	public JComponent getComponent()
	{
		return this;
	}
	
   private final class ValidationListMouseAdapter extends MouseAdapter
	{
		private final JList list;
		private final JXmlTextArea textArea;

		public ValidationListMouseAdapter(JList list, JXmlTextArea textArea)
		{
			this.list = list;
			this.textArea = textArea;
		}

		public void mouseClicked(MouseEvent e)
		{
		   if( e.getClickCount() < 2 ) return;
		
		   int ix = list.getSelectedIndex();
		   if( ix == -1 ) return;
		   
		   Object obj = list.getModel().getElementAt( ix );
		   if( obj instanceof AssertionError )
		   {
		   	AssertionError error = (AssertionError) obj;
		   	if( error.getLineNumber() >= 0 )
		   	{
		   		textArea.setCaretPosition( textArea.getLineStartOffset( error.getLineNumber()-1 ) );
		   		textArea.requestFocus();
		   	}
		   	else Toolkit.getDefaultToolkit().beep();
		   }
		   else Toolkit.getDefaultToolkit().beep();
		}
	}
   
   public boolean dependsOn(ModelItem modelItem)
	{
		return modelItem == request || modelItem == requestStep || modelItem == request.getOperation() ||
		    modelItem == request.getOperation().getInterface() || 
		    modelItem == request.getOperation().getInterface().getProject() ||
		    modelItem == request.getTestCase() || modelItem == request.getTestCase().getTestSuite();
	}
}
