/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.request.actions;

import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JSplitPane;
import javax.swing.KeyStroke;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.WsdlOperation;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.support.WsdlValidator;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError;
import com.eviware.soapui.support.JXmlTextArea;

/**
 * Validates the request XML of a WsdlRequest
 * 
 * @author Ole.Matzura
 */

public class ValidateRequestXmlAction extends AbstractAction
{
	private final JComponent container;
	private final DefaultListModel errorListModel;
	private final JSplitPane splitPane;
	private final JXmlTextArea textArea;
	private final WsdlRequest request;

	public ValidateRequestXmlAction(WsdlRequest request, JComponent container, DefaultListModel errorListModel, JSplitPane splitPane, JXmlTextArea textArea )
	{
		super( "Validate" );
		putValue( Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke( KeyEvent.VK_V, KeyEvent.ALT_MASK ));
		
		this.request = request;
		this.container = container;
		this.errorListModel = errorListModel;
		this.splitPane = splitPane;
		this.textArea = textArea;
	}

	public void actionPerformed(ActionEvent e)
	{
	   WsdlOperation operation = (WsdlOperation) request.getOperation();
		WsdlValidator validator = new WsdlValidator( ((WsdlInterface)operation.getInterface()).getWsdlContext() );
	   
		AssertionError[] errors = validator.assertRequest( textArea.getText(), operation.getBindingOperationName() );
		
		errorListModel.clear();
	   if( errors == null || errors.length == 0 )
	   {
	   	splitPane.setDividerLocation( 1.0 );
	   	JOptionPane.showMessageDialog( SoapUI.getInstance().getFrame(), "XML is valid", "Validation",
	   			JOptionPane.INFORMATION_MESSAGE );
		   container.setVisible( false );
	   }
	   else
	   {
	   	Toolkit.getDefaultToolkit().beep();
		   for( int c = 0; c < errors.length; c++ )
		   {
		   	errorListModel.addElement( errors[c] );
		   }
		   container.setVisible( true );
		   splitPane.setDividerLocation( 0.8 );
	   }
	}
}