/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps.assertions;

import javax.swing.JOptionPane;

import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlObject;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.RequestAssertionConfig;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlAssertion;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequest;

/**
 * Assertion that checks for the non-existence of a specified text token in the associated 
 * WsdlTestRequests response message
 * 
 * @author Ole.Matzura
 */

public class SimpleNotContainsAssertion extends WsdlAssertion
{
   private String token;

   public SimpleNotContainsAssertion(RequestAssertionConfig assertionConfig, WsdlTestRequest request)
   {
      super(assertionConfig, request);
      
      XmlObject[] paths = getConfiguration().selectPath( "$this/token" );
      if( paths.length != 1 ) return;
         
      token = paths[0].newCursor().getTextValue();
   }
   
   public String assertRequest(WsdlTestRequest request) throws AssertionException
   {
      String response = request.getResponseContent();
      if( token == null ) token = "";

      if( token.length() > 0 && response.indexOf( token ) != -1 )
      {
         throw new AssertionException( new AssertionError("Response contains token [" + token + "]") );
      }
      
      return "Response does not contain token [" + token + "]";
   }

   public void configure()
   {
      String value = (String) JOptionPane.showInputDialog( SoapUI.getInstance().getFrame(), 
      		"Configure SimpleNotContains Assertion", "Specify token to be missing",
      		JOptionPane.QUESTION_MESSAGE, null, null, token );
      
      if( value == null || value.equals( token )) return;
      token = value;
      
      setConfiguration( createConfiguration() );
   }

   protected XmlObject createConfiguration()
   {
      XmlObject config = XmlObject.Factory.newInstance();
      XmlCursor cursor = config.newCursor();
      cursor.toNextToken();
      cursor.beginElement( "token" );
      cursor.insertChars( token );
      cursor.dispose();
      return config;
   }

   public boolean isConfigurable()
   {
      return true;
   }
}
