/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.model.tree;

import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.Action;
import javax.swing.JPopupMenu;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.model.DesktopPanel;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.PanelBuilder;
import com.eviware.soapui.model.testsuite.TestCase;

/**
 * Abstract base class for SoapUITreeNode implementations
 * 
 * @author Ole.Matzura
 */

public abstract class AbstractTreeNode implements SoapUITreeNode, PropertyChangeListener
{
   private final ModelItem modelItem;
   private JPopupMenu popup;
   private PanelBuilder panelBuilder;
   private SoapUITreeNode parent;

   protected AbstractTreeNode( ModelItem modelItem, ModelItem parentItem )
   {
      this.modelItem = modelItem;
      if( parentItem != null )
         this.parent = parentItem.getTreeNode();
      
      initPopup();
      
      panelBuilder = modelItem.getPanelBuilder();
      
      modelItem.addPropertyChangeListener( TestCase.NAME_PROPERTY, this );
      modelItem.addPropertyChangeListener( TestCase.ICON_PROPERTY, this );
   }
   
   public SoapUITreeModel getTreeModel()
   {
      return SoapUI.getInstance().getTreeModel();
   }

   public boolean hasContentPanel()
   {
      return panelBuilder.canBuildDesktopPanel();
   }
   
   public Component getOverviewPanel()
   {
      return panelBuilder.buildOverviewPanel();
   }
   
   public DesktopPanel getDesktopPanel()
   {
      return panelBuilder.buildDesktopPanel();
   }

   public ModelItem getModelItem()
   {
      return modelItem;
   }

   public boolean valueChanged(Object newValue)
   {
      return false;
   }

   public boolean isLeaf()
   {
      return getChildCount() == 0;
   }

   public int getChildCount()
   {
      return 0;
   }

   public SoapUITreeNode getChildNode(int index)
   {
      return null;
   }

   public int getIndexOfChild(Object child)
   {
      return -1;
   }
   
   public String toString()
   {
      return modelItem.getName();
   }
   
   public JPopupMenu getPopup()
   {
      return popup;
   }
   
   public SoapUITreeNode getParent()
   {
      return parent;
   }

   protected void initPopup()
   {
      Action[] actions = modelItem.getActions();
      if( actions == null || actions.length == 0 ) return;
      
      popup = new JPopupMenu();
      
      for (int i = 0; i < actions.length; i++)
      {
         popup.add( actions[i] );
      }
   }
   
   public void setParentNode(SoapUITreeNode parent)
   {
      this.parent = parent;
   }

   public void propertyChange(PropertyChangeEvent evt)
   {
      getTreeModel().notifyNodeChanged( this );
   }
   
   public void release()
   {
   	modelItem.removePropertyChangeListener( this );
   }
}
