/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.model.tree.nodes;

import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.model.testsuite.TestSuite;
import com.eviware.soapui.model.testsuite.TestSuiteListener;
import com.eviware.soapui.model.tree.AbstractTreeNode;
import com.eviware.soapui.model.tree.SoapUITreeNode;

/**
 * SoapUITreeNode for TestSuite implementations
 * 
 * @author Ole.Matzura
 */

public class TestSuiteTreeNode extends AbstractTreeNode
{
   private InternalTestSuiteListener internalTestSuiteListener;

	public TestSuiteTreeNode(TestSuite testSuite )
   {
      super( testSuite, testSuite.getProject() );
      
      internalTestSuiteListener = new InternalTestSuiteListener();
		testSuite.addTestSuiteListener( internalTestSuiteListener );
   }

	public void release()
	{
		((TestSuite)getModelItem()).removeTestSuiteListener( internalTestSuiteListener );
	}
	
   public int getChildCount()
   {
     return getTestSuite().getTestCaseCount();
   }

   public TestSuite getTestSuite()
   {
      return (TestSuite) getModelItem();
   }
   
   public int getIndexOfChild(Object child)
   {
      TestSuite testSuite = getTestSuite();
		for( int c = 0; c < testSuite.getTestCaseCount(); c++ )
      {
         if( testSuite.getTestCaseAt( c ).getTreeNode() == child ) return c;
      }
      
      return -1;
   }

   public SoapUITreeNode getChildNode(int index)
   {
      return getTestSuite().getTestCaseAt( index ).getTreeNode();
   }   
   
   private class InternalTestSuiteListener extends InternalTreeNodeListener implements TestSuiteListener
   {
   	public InternalTestSuiteListener()
   	{
   		super( getTreeModel() );
   	}
   	
      public void testCaseAdded(TestCase testCase)
      {
         notifyNodeInserted( testCase );
      }

      public void testCaseRemoved(TestCase testCase)
      {
         notifyNodeRemoved( testCase );
      }

      public void testStepAdded(TestStep testStep, int index)
      {
         notifyNodeInserted( testStep );
      }

      public void testStepRemoved(TestStep testStep, int ix)
      {
         notifyNodeRemoved( testStep );
      }
   }
}
